from functools import cached_property


def stateful_property(default_value=None, constructor=None):
    class _stateful_property(cached_property):
        def __init__(self, func):
            super().__init__(constructor or (lambda self: default_value))
            self.setter = func

        def __set__(self, instance, value):
            if instance is None:
                return self
            if self.attrname is None:
                raise TypeError(
                    "Cannot use cached_property instance without calling __set_name__ on it.")
            try:
                cache = instance.__dict__
            except AttributeError:  # not all objects have __dict__ (e.g. class defines slots)
                msg = (
                    f"No '__dict__' attribute on {type(instance).__name__!r} "
                    f"instance to cache {self.attrname!r} property."
                )
                raise TypeError(msg) from None
            with self.lock:
                # check if another thread filled cache while we awaited lock
                val = self.setter(instance, value)
                try:
                    cache[self.attrname] = val
                except TypeError:
                    msg = (
                        f"The '__dict__' attribute on {type(instance).__name__!r} instance "
                        f"does not support item assignment for caching {self.attrname!r} property."
                    )
                    raise TypeError(msg) from None

    return _stateful_property



'''
Find and locate resources for the application.

Attributes:
    PATHS (List[str]): Various paths where the application data may be
        stored at.
        Note: orders matter, especially when the program is installed
        globally.

'''
from . import _, logger, PACKAGE

import os
try:
    import xdg.BaseDirectory
    import xdg.IconTheme
except ImportError:
    class xdg:
        class BaseDirectory:
            @staticmethod
            def load_data_paths(resource):
                yield '/usr/share/' + resource

        class IconTheme:
            @staticmethod
            def getIconPath(iconname):
                path = f'/usr/share/icons/hicolor/48x48/apps/{iconname}.png'
                if os.path.isfile(path):
                    return path


__all__ = ['get_data_path', 'get_icon_path', 'LOGO_PATH']


PATHS = [
    os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources'),
    os.path.join(os.path.dirname(
        os.path.dirname(os.path.realpath(__file__))), 'resources'),
    *xdg.BaseDirectory.load_data_paths(PACKAGE)
]


def get_data_path(filenames, is_dir=False):
    '''
    Find a file or directory by its name in multiple locations.

    It will search the target in the directory from `PATHS` sequentially,
    and return the first item matches. If none matches, ``None`` is
    returned.

    Args:
        filenames (Union[str, List[str]]): Filename(s) of the target. If
            multiple filename provides, the first one will be used as
            'master' one shown in the log.
        is_dir (bool, optional): Whether the target filename represents a
            directory. Default is false.

    Returns:
        Optional[str]: The path of the target.

    '''
    if type(filenames) != str:
        filename = filenames[0]
    else:
        filename = filenames
        filenames = (filename, )
    for test_location in PATHS:
        for alt_filename in filenames:
            candidate = os.path.join(test_location, alt_filename)
            if os.path.isdir(candidate) if is_dir \
                    else os.path.isfile(candidate):
                logger.debug('locate "{}" at {}'.format(filename, candidate))
                return candidate
    logger.warn('locate "{}" FAILED'.format(filename))


def get_icon_path(iconname):
    path = xdg.IconTheme.getIconPath(iconname)
    if path:
        return path
    for extensions in ['png', 'svg', 'xpm']:
        path = get_data_path('icons/{}.{}'.format(iconname, extensions))
        if path:
            return path


LOGO_PATH = get_icon_path(PACKAGE)
if LOGO_PATH is None:
    logger.warn(_('Logo missing. Is your installation correct?'))



'''
Shared components and constants of the application.

Attributes:
    PACKAGE: The package name.
    APPID: Appid for Gtk.Application.
    INFO: Other info (version, url, etc.) of the application.
        Also used in Gtk.AboutDialog construction, unknown parameter may
        cause error.
    _: Gettext function.
    logger: Logger for debug and messages.

'''
try:
    from .version import VERSION
except ImportError:
    VERSION = 'git'

import gettext
import logging
import sys


PACKAGE = 'galternatives'

_ = gettext.gettext
gettext.bindtextdomain(PACKAGE)
gettext.textdomain(PACKAGE)

logger = logging.getLogger(PACKAGE)

# set gtk version for the whole module
try:
    import gi
    gi.require_version('Gdk', '3.0')
    gi.require_version('GdkPixbuf', '2.0')
    gi.require_version('Gtk', '3.0')
except (ImportError, AttributeError):
    # in some cases gi is not installed since we only want to get INFO
    logger.warn('gi is not installed, assuming you only want to get info for '
                'this application.')

APPID = 'org.debian.' + PACKAGE
INFO = {
    'program_name': 'G Alternatives',
    'version': VERSION,
    'comments': _('A tool to help the administrator select which programs '
                  'provide specific services for the user by default.'),
    'license_type': 'GPL_2_0',
    'copyright': '''(C) 2003-2006 Gustavo Noronha Silva
(C) 2017-2019 Boyuan Yang''',
    'website': 'https://salsa.debian.org/chinese-team/galternatives',
    'authors': (
        'Gustavo Noronha Silva <kov@debian.org>',
        'Boyuan Yang <073plan@gmail.com>',
        'Yangfl <mmyangfl@gmail.com>',
    ),
    'artists': (
        'Leandro A. F. Pereira <leandro@linuxmag.com.br>',
    ),
}


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)



from . import logger, eprint, _, PACKAGE, APPID, alternative
from .appdata import *
from .gui import MainWindow, AboutDialog
try:
    from .log import set_logger
except ImportError:
    set_logger = None

from functools import cached_property
from gi.repository import Gio, GLib, Gtk
import shutil
import os


class GAlternativesApp(Gtk.Application):
    use_polkit = False
    target_group = None

    def __init__(self, *args, **kwargs):
        super().__init__(
            *args, application_id=APPID,
            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
            **kwargs)
        self.add_main_option(
            'debug', ord('d'), GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
            _('Enable debug output'), None)
        self.add_main_option(
            'normal', ord('n'), GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
            _('Do not try to acquire root (as normal user)'), None)
        self.add_main_option(
            'altdir', 0, GLib.OptionFlags.NONE, GLib.OptionArg.FILENAME,
            _('Specify the alternatives directory'), None)
        self.add_main_option(
            'admindir', 0, GLib.OptionFlags.NONE, GLib.OptionArg.FILENAME,
            _('Specify the administrative directory'), None)
        self.add_main_option(
            'log', 0, GLib.OptionFlags.NONE, GLib.OptionArg.FILENAME,
            _('Specify the log file'), None)
        # BUG: Gtk.Application.add_option_group() not working

    def do_handle_local_options(self, options):
        if not self.debug:
            self.debug = bool(options.contains('debug'))
        if set_logger:
            set_logger(PACKAGE, self.debug)
            logger.debug(_('Testing galternatives...'))

        # GtkBuilder will error out if the version requirements are not met.
        # No need to check it again.

        if os.getuid():
            # not root
            if options.contains('normal'):
                logger.warn(_(
                    'No root privileges detected, but continuing anyway'))
            elif shutil.which('pkexec'):
                self.use_polkit = True
            else:
                dialog = Gtk.MessageDialog(
                    None, Gtk.DialogFlags.DESTROY_WITH_PARENT,
                    Gtk.MessageType.WARNING, Gtk.ButtonsType.OK_CANCEL,
                    _('<b><tt>pkexec</tt> required for privileged '
                      'operations.</b>'),
                    use_markup=True,
                    secondary_text=_(
                        'The program needs pkexec to perform privileged '
                        'alternatives system modifications under normal user. '
                        'Unless you have modified your system to explicitly '
                        'allow your normal user to do so, GAlternatives will '
                        'not work.'))
                if dialog.run() != Gtk.ResponseType.OK:
                    return 1
                dialog.destroy()

        self.paths = {}
        for option in alternative.Alternative.PATHS:
            value = options.lookup_value(option)
            if value:
                value = value.unpack()
                value.pop()
                self.paths[option] = bytearray(value).decode('utf-8')

        return -1

    def do_local_command_line(self, arguments):
        self.debug = False
        for i, arg in enumerate(arguments):
            if arg.startswith('-dd'):
                self.debug = 2
                break
        return Gtk.Application.do_local_command_line(self, arguments)

    def do_command_line(self, command_line):
        args = command_line.get_arguments()
        if len(args) > 2:
            eprint(_('Specifying more than one group not allowed'))
            return 2
        if len(args) > 1:
            self.target_group = args[1]
            if self.target_group not in self.window.alt_db:
                eprint(_('No such group'))
                self.window.destroy()
                return 1
        self.activate()
        return 0

    def do_startup(self):
        Gtk.Application.do_startup(self)
        self.set_app_menu(
            Gtk.Builder.new_from_file(get_data_path('glade/menubar.ui'))
            .get_object('menu'))

        # Cannot use add_action_entries()
        # see https://bugzilla.gnome.org/show_bug.cgi?id=678655
        for name, activate in {
            'about': self.on_about,
        }.items():
            action = Gio.SimpleAction(name=name)
            action.connect('activate', activate)
            self.add_action(action)

    def do_activate(self):
        if self.debug > 1:
            self.window.edit_warning_show_check.set_active(False)
        self.window.show()

    def on_about(self, action, param):
        self.about_dialog.present()

    @cached_property
    def about_dialog(self):
        return AboutDialog(
            transient_for=self.window and self.window.main_window)

    @cached_property
    def window(self):
        return MainWindow(self, self.paths, self.target_group)



'''
Give readable information for various internal objects.

Attributes:
    DESC_DIR (str): Directory path where descriptions for alternative groups
        are stores.
    DEFAULT_DESCRIPTION (str): Placeholder when no description is found.

'''
from . import logger, _
from .appdata import *

import configparser
import locale
import subprocess
import sys
import os


__all__ = ['altname_description', 'query_package', 'friendlize']


DESC_DIR = get_data_path('descriptions', True)
DEFAULT_DESCRIPTION = _('No description')
DEFAULT_ICON = 'dialog-question'


def altname_description(name, locale=locale.getdefaultlocale()[0]):
    '''
    Find readable group name and its description for given alternative group
    name.

    It will try to find proper translated strings, then the untranslated
    name and description. If both failed, the original name will be
    returned, along with a placeholder 'No description' as its description.

    Args:
        name (str): Name of the group.
        locale (str, optional): Preferred locale. Default to current
            environment locale.

    Returns:
        Tuple[str, str]: Readable group name and its description.

    '''
    desc_file = os.path.join(DESC_DIR, '{}.desktop'.format(name))
    config = configparser.ConfigParser()
    # this func call also read the config file!
    if desc_file in config.read(desc_file):
        section = config['Desktop Entry']
        return (
            section.get('Name[{}]'.format(locale)) or
                section.get('Name') or
                name,
            section.get('Comment[{}]'.format(locale)) or
                section.get('Comment') or
                DEFAULT_DESCRIPTION,
            section.get('Icon') or
                DEFAULT_ICON,
        )
    return (name, DEFAULT_DESCRIPTION, DEFAULT_ICON)


def query_package(filename):
    '''
    Query which package provides the file.

    If no package matches, an empty string will be returned. If multiple package
    match, the first result will be returned.

    In order to make the best guess, use the absolute path of the target file.

    Args:
        filename (str): Path of the file.

    Returns:
        str: The package name which provides it.

    Raises:
        RuntimeError: If ``dpkg`` exits with non-zero code.

    '''
    try:
        p = subprocess.run(['dpkg', '-S', filename], capture_output=True)
    except subprocess.CalledProcessError:
        raise RuntimeError(f"`dpkg' returned with code {p.returncode}")
    if p.stdout:
        return p.stdout.decode().split(': ')[0]


def friendlize(commands):
    '''
    Convert ``update-alternatives`` commands into readable descriptions.

    Args:
        commands (List[str]): List of commands from
            ``Alternative.compare()``

    Yields:
        List[str]: The readable description for the command.

    '''
    for cmd in commands:
        type_ = cmd[0]
        if type_ == 'install':
            yield (
                _("Install option `{3}' for group `{2}'").format(*cmd),
                _('Priority: {4}').format(*cmd),
                *(_("Slave `{}': `{}'").format(name, path)
                  for __, __, name, path in zip(*((iter(cmd[5:]), ) * 4))),
            )
        elif type_ == 'auto':
            yield (
                _("Set group `{1}' to auto mode").format(*cmd),
            )
        elif type_ == 'set':
            yield (
                _("Set group `{1}' to manual mode, pointed to `{2}'")
                .format(*cmd),
            )
        elif type_ == 'remove':
            yield (
                _("Remove option `{2}' for group `{1}'").format(*cmd), )
        elif type_ == 'remove-all':
            yield (
                _("Remove group `{1}'").format(*cmd),
            )



'''
Set formatter for logger.

This file should only be imported in the main function. It can be safely
removed.
'''
from copy import copy
import logging


# The background is set with 40 plus the number of the color, and the foreground
# with 30
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

# These are the sequences need to get colored output
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;{}m"
BOLD_SEQ = "\033[1m"
COLORS = {
    'WARNING': YELLOW,
    'INFO': WHITE,
    'DEBUG': BLUE,
    'CRITICAL': YELLOW,
    'ERROR': RED
}


class ColoredFormatter(logging.Formatter):
    def format(self, record):
        '''
        Format the record using the underlying formatter, but display the
        level name in color.

        The function does not change the original record.
        '''
        record = copy(record)
        levelname = record.levelname
        if levelname in COLORS:
            record.levelname = ''.join((
                COLOR_SEQ.format(30 + COLORS[levelname]), levelname, RESET_SEQ))
        return super().format(record)


def set_logger(ns, verbose=False):
    '''
    Set console output for logger of giver namespace.

    Args:
        ns (str): Namespace of the logger.
        verbose (bool, optional): Whether to set logger level to debug.
            Default is false.

    '''
    logger = logging.getLogger(ns)
    if verbose:
        logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setFormatter(ColoredFormatter(
        # log by time
        # '%(asctime)s - %(levelname)s: %(message)s'))
        # gtk style
        # '(%(filename)s:%(process)d): %(funcName)s-%(levelname)s **: %(message)s'))
        # gtk style but lineno
        '(%(filename)s->%(lineno)d): %(funcName)s-%(levelname)s **: %(message)s'))
    logger.addHandler(ch)



'''
Handle alternative database.

The library can read the database of alternative system, make virtual
changes to it, and output ``update-alternatives`` commands according to the
differences between two databases, while saving the state of the original
database is user's duty, by calling ``deepcopy(db)``.

The difference is represented in a list of ``update-alternatives`` argument
lists, without the leading two dashes.

Note we do not try to implement a full version of ``update-alternatives`` in
Python; format and mechanism of alternative system may change in the future,
while we'd like to describe the most fundamental features that will not
likely to be changed.

'''
from . import logger

from collections import namedtuple
import os
import subprocess
import sys


def except_typeof(i, of_type):
    if not isinstance(i, of_type):
        raise TypeError('instance "{}" not of type {}, got {}'.format(
            repr(i), of_type.__name__, type(i).__name__))


def except_absolute(path):
    if path == '':
        return
    except_typeof(path, str)
    if not os.path.isabs(path):
        raise ValueError('path not absolute')


def except_tag(tag):
    except_typeof(tag, str)
    if any(char in tag for char in ' \t\n\r'):
        raise ValueError('special char not allowed')


CommandResult = namedtuple('CommandResult', ('cmd', 'returncode', 'out', 'err'))


class Option(dict):
    '''
    Class for option of alternative group.

    The class does not remember the group it belongs to; it's more like a
    wrapper for dict.

    Attributes:
        priority (int): Priority of the option.

    '''
    def __init__(self, it=(), priority=0, *args, **kwargs):
        '''
        Init the class.

        Args:
            it (Iterable, optional): Iterator for dict initialization. Default
                is empty list.
            priority (int, optional): Priority of the option. Default is 0.
            *args: Variable length argument list will be passed to the super
                constructor.
            **kwargs: Arbitrary keyword arguments will be passed to the super
                constructor.

        '''
        super().__init__(it, *args, **kwargs)
        self.priority = priority

    def describe(self, group):
        '''
        Convert to ``update-alternatives`` command.

        Args:
            group (Group): The group which it belongs to.

        Returns:
            Tuple[str, ...]: ``update-alternatives`` command.

        Raises:
            ValueError: If the master path is missing.

        '''
        diff = []
        for ind, name in enumerate(group):
            if name in self and self[name]:
                diff.extend((
                    '--slave' if ind else 'install', group[name],
                    name, self[name]))
            elif ind == 0:
                raise ValueError('master path missing')
            if ind == 0:
                diff.append(str(self.priority))
        return tuple(diff)

    def same_with(self, other, group):
        '''
        Tell whether two options have the same path set for the group.

        Args:
            other (Option): Option to be compared with.
            group (Group): The group which it belongs to.

        Returns:
            bool: The result.

        '''
        return all(self[name] == other[name] for name in group)

    def __hash__(self):
        return id(self)

    def paths(self, group):
        return tuple(
            self[group[i]] if group[i] in self else ''
            for i in range(len(group)))


class Group(list):
    '''
    Class for alternative group.

    It acts like an OrderedDict, but allows int as index. The class itself
    stores the names for the master and slave links, while the actual links are
    stored in `_links` attribute.

    See update-alternatives(1) for properties' detail.

    Attributes:
        _parent (Alternative): The associated alternative database.
        _links (Dict[str, str]): Links for names.
        options (List[Option]): Available options.
        status (bool): Whether the group is in auto mode. Read only.
        current (Option): The current applied option. Read only.

    '''
    def __init__(self, name, create=False, parent=None):
        super().__init__()
        self._parent = parent
        self._links = {}
        self._current = True
        self[name] = ''
        self.options = []
        if not create:
            self.reload()

    def reload(self):
        if not os.path.isfile(self.alt):
            raise IOError('group alt file missing')

        # parse alt file
        with open(self.alt) as alt_file:
            it = map(lambda s: s.strip(), alt_file)

            # target parser
            self._current = next(it) == 'auto'
            self[self.name] = next(it)
            while True:
                line = next(it)
                if line == '':
                    break
                self[line] = next(it)

            # option parser
            while True:
                it_name = iter(self)
                line = next(it)
                if line == '':
                    break
                option = Option({next(it_name): line}, int(next(it)))
                for name in it_name:
                    option[name] = next(it)
                self.options.append(option)

        admin = os.path.join(self._parent.admindir, self.name)
        if not os.path.islink(admin):
            raise IOError('admin link for "{}" corrupted'.format(name))
        current_path = os.readlink(admin)

        if not self._current:
            for option in self.options:
                if option[self.name] == current_path:
                    self._current = option

    @property
    def alt(self):
        '''Path of alt file in the admindir.'''
        return os.path.join(self._parent.altdir, self.name)

    @property
    def name(self):
        '''Name of the group.'''
        return self[0]

    @property
    def link(self):
        '''Master link of the group.'''
        return self[self.name]

    @link.setter
    def link(self, value):
        self[self.name] = value

    @property
    def status(self):
        return self._current is True

    @property
    def current(self):
        return self.best if self.status else self._current

    @property
    def best(self):
        '''The best option for the group (the one with the highest priority).'''
        return max(self.options, key=lambda o: o.priority)

    @property
    def options_dict(self):
        res = {option[self.name]: option for option in self.options}
        if len(res) != len(self.options):
            raise ValueError('duplicated master path found')
        return res

    def select(self, index=None):
        if index is None:
            if self.status:
                return False
            else:
                self._current = True
                return True
        else:
            if not self.status and self.current == self.options[index]:
                return False
            else:
                self._current = self.options[index]
                return True

    def __setitem__(self, index, value):
        if type(index) == int:
            if value in self._links:
                raise KeyError('element already exists')
            # except_tag(value)
            self._links[value] = self._links[self[index]]
            del self._links[self[index]]
            super().__setitem__(index, value)
        else:
            # except_absolute(value)
            if index not in self._links:
                super().append(index)
            self._links[index] = value

    def __getitem__(self, index):
        if type(index) in (int, slice):
            return super().__getitem__(index)
        else:
            return self._links[index]

    def __delitem__(self, index):
        if type(index) != int:
            index = self.index(index)
        del self._links[super().__getitem__(index)]
        super().__delitem__(index)

    def __repr__(self):
        return 'altgroup: {} links: {} options: {}'.format(
            self.name, repr(self._links), self.options)

    def __hash__(self):
        return id(self)

    def items(self):
        return [(link, self[link]) for link in self]

    def find_option(self, path):
        for option in self.options:
            if option[self.name] == path:
                return option

    def compare(self, old_group={}):
        '''
        Compare two groups and return the differential commands.

        Args:
            old_group (Group, optional): Original group to be compared with.

        Returns:
            List[Tuple[str, ...]]: Differential commands.

        '''
        new_options = self.options_dict
        old_options = old_group and old_group.options_dict
        res = [
            ('remove', self.name, p)
            for p in old_options
            if p not in new_options
        ] + [
            o.describe(self)
            for p, o in new_options.items()
            if p not in old_options or not o.same_with(old_options[p], self)
        ]
        if self.status:
            # self in auto
            if old_group and not old_group.status:
                # old in manual
                res.append(('auto', self.name))
        else:
            # self in manual
            if not old_group or old_group.status or \
                    not self.current.same_with(old_group.current, self):
                # no old, old in auto, old current not the same
                res.append(('set', self.name, self.current[self.name]))
        return res


def list_starts_with(l, prefix):
    return l[:len(prefix)] == prefix


class Alternative(dict):
    altdir = '/var/lib/dpkg/alternatives'
    admindir = '/etc/alternatives'
    log = '/var/log/alternatives.log'
    update_alternatives = 'update-alternatives'
    PATHS = ('altdir', 'admindir', 'log')

    def __init__(self, altdir=None, admindir=None, log=None):
        if altdir is not None:
            self.altdir = altdir
        if admindir is not None:
            self.admindir = admindir
        if log is not None:
            self.log = log
        self._moves = {}
        super().__init__(
            map(lambda name: (name, None), filter(
                lambda name: os.path.isfile(os.path.join(self.altdir, name)),
                os.listdir(self.altdir))))

    def __repr__(self):
        return repr(self.keys())

    def __getitem__(self, name):
        item = super().__getitem__(name)
        if not item:
            item = Group(name, parent=self)
            super().__setitem__(name, item)
        return item

    def add(self, item):
        '''
        Add new group to the alternative database.

        Args:
            item (Group): Group to be added.

        Raises:
            KeyError: If the group name already exists.

        '''
        # except_typeof(item, Group)
        if item.name in self:
            raise KeyError('element already exists')
        item._parent = self
        self._moves[item.name] = None
        return super().__setitem__(item.name, item)

    def move(self, old, new):
        if old in self._moves:
            self._moves[new] = self._moves[old]
            del self._moves[old]
        else:
            self._moves[new] = old
        if self._moves[new] == new:
            del self._moves[new]
        self[old][0] = new
        super().__setitem__(new, self[old])
        super().__delitem__(old)

    def compare(self, old_db):
        diff = []
        for group_name in old_db:
            if group_name not in self:
                # group in old but not in new
                diff.append(('remove-all', group_name))
        for group_name in self:
            if dict.__getitem__(self, group_name):
                # item has been accessed
                if group_name in self._moves or \
                        not list_starts_with(self[group_name].items(),
                                             old_db[group_name].items()):
                    # group moved or more links than old one, reconstruct
                    if group_name in self._moves and \
                            self._moves[group_name] in old_db and \
                            self._moves[group_name] in self:
                        diff.append(('remove-all', self._moves[group_name]))
                    elif group_name in old_db:
                        diff.append(('remove-all', group_name))
                    diff.extend(self[group_name].compare())
                else:
                    # simply add difference
                    diff.extend(self[group_name].compare(old_db[group_name]))
        return diff

    def commit(self, diff, executer=None):
        results = []
        for cmd in diff:
            args = list(cmd)
            args[0] = '--' + args[0]
            args.insert(0, self.update_alternatives)
            if executer:
                args.insert(0, executer)
            logger.debug(f'run command {args}')
            p = subprocess.run(args, capture_output=True, text=True)
            results.append(CommandResult(
                args, p.returncode, p.stdout, p.stderr))
            if p.returncode:
                break
        return p.returncode, results



#!/usr/bin/env python3
import os
import sys
# If run as a single file (rather than a module), include the correct path so
# that the package can be imported
if __name__ == '__main__' and __package__ is None:
    sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from galternatives.app import GAlternativesApp

import signal


def main():
    # Allow Ctrl-C to work
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    app = GAlternativesApp()
    return app.run(sys.argv)


if __name__ == '__main__':
    sys.exit(main())



'''
Interface of the application.
'''
from . import logger, _, PACKAGE, INFO, alternative
from .appdata import *
from .description import *
from .utils import stateful_property

from copy import deepcopy
from functools import wraps
from gi.repository import GdkPixbuf, Gdk, Gio, GLib, Gtk, GObject
import os
import shutil
import sys
import threading
try:
    import xdg.Config
except ImportError:
    class xdg:
        class Config:
            icon_size = 48
from itertools import zip_longest

GObject.threads_init()


def hide_on_delete(window, *args):
    '''
    Warpper for Gtk.Widget.hide_on_delete, but allow superfluous arguments.
    Used for signal callback.
    '''
    return Gtk.Widget.hide_on_delete(window)


def reset_dialog(dialog, *args):
    '''
    Select cancel button as default when reshow the dialog. Used for signal
    callback.
    '''
    btn_cancel = \
        dialog.get_widget_for_response(Gtk.ResponseType.CANCEL) or \
        dialog.get_widget_for_response(Gtk.ResponseType.CLOSE)
    btn_cancel.grab_focus()
    btn_cancel.grab_default()


@Gtk.Template.from_file(get_data_path('glade/file_entry.glade'))
class FileEntry(Gtk.Box):
    __gtype_name__ = 'FileEntry'
    entry = Gtk.Template.Child('entry')

    @Gtk.Template.Callback('open_file')
    def open_file(self, button):
        '''Select a file and fill the entry with the path.'''
        file_chooser = Gtk.FileChooserDialog(
            title=_('Select File'), action=Gtk.FileChooserAction.OPEN,
            parent=button.get_toplevel(), buttons=(
                Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        if file_chooser.run() == Gtk.ResponseType.OK:
            self.entry.set_text(file_chooser.get_filename())
        file_chooser.destroy()

    def set_text(self, text):
        return self.entry.set_text(text)

    def get_text(self):
        return self.entry.get_text()

    def set_icon_from_icon_name(self, icon_pos, icon_name):
        return self.entry.set_icon_from_icon_name(icon_pos, icon_name)

    def connect(self, detailed_signal, handler):
        return self.entry.connect(
            detailed_signal, lambda widget, *args: handler(self, *args))


# can't inherit GtkTemplate, need workaround

@Gtk.Template.from_file(get_data_path('glade/edit_dialog.glade'))
class EditDialog(Gtk.Dialog):
    __gtype_name__ = 'EditDialog'
    slave_it = None

    requires = Gtk.Template.Child('requires')
    slaves_tv = Gtk.Template.Child('slaves_tv')
    slave_fields = Gtk.Template.Child('slave_fields')
    slaves_edit = Gtk.Template.Child('slaves_edit')
    new_group_warning = Gtk.Template.Child('new_group_warning')

    def _init_edit_dialog(self):
        for i, (field_name, label_name, widget_class) in \
                enumerate(self.REQUIRES):
            widget = widget_class()
            widget.set_hexpand(True)
            setattr(self, field_name.lower(), widget)
            self.requires.attach(Gtk.Label(label=label_name), 0, i, 1, 1)
            self.requires.attach(widget, 1, i, 1, 1)
        self.requires.show_all()

        self.slaves_entries = []
        for i, (column_name, label_name, widget_class) in \
                enumerate(self.SLAVES):
            # slaves_tv
            column = Gtk.TreeViewColumn(
                label_name, Gtk.CellRendererText(),
                text=2 * i, background=2 * i + 1)
            column.set_resizable(True)
            self.slaves_tv.append_column(column)
            # slave_fields
            widget = widget_class()
            widget.i_column = i
            widget.connect('changed', self.on_slave_fields_changed)
            self.slaves_entries.append(widget)
            box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
            box.pack_start(Gtk.Label(label=label_name), False, False, 0)
            box.pack_start(widget, True, True, 0)
            self.slave_fields.pack_start(box, False, False, 0)
        self.slave_fields.show_all()
        self.slaves_model = Gtk.ListStore(*(str, ) * len(self.SLAVES) * 2)
        self.slaves_tv.set_model(self.slaves_model)

    def __new__(cls, *args, **kwargs):
        if cls != EditDialog:
            self = EditDialog(**kwargs)
            self.__class__ = cls
            EditDialog._init_edit_dialog(self)
            return self
        else:
            return super().__new__(cls, *args, **kwargs)

    @Gtk.Template.Callback('add_row')
    def add_row(self, button):
        self.slaves_model.append((None, ) * len(self.SLAVES) * 2)

    @Gtk.Template.Callback('remove_row')
    def remove_row(self, button):
        model, it = self.slaves_tv.get_selection().get_selected()
        del model[it]
        for widget in self.slaves_entries:
            widget.set_text('')

    @Gtk.Template.Callback('on_click_slave')
    def on_click_slave(self, widget):
        model, it = widget.get_selected()
        self.slave_it = it
        if it is None:
            return
        for widget, text in zip_longest(
                self.slaves_entries, model[it][::2] if it else ()):
            widget.set_text(text or '')

    def on_slave_fields_changed(self, widget):
        if self.slave_it is None:
            return
        self.slaves_model[self.slave_it][2 * widget.i_column] = \
            widget.get_text()

    @Gtk.Template.Callback('on_response')
    def on_response(self, window, response_id):
        # only bind to cancel button
        # dialog response can not be cancelled, thus not suitable for validation
        if response_id == Gtk.ResponseType.CANCEL:
            # do not emit delete-event
            self.destroy()

    @Gtk.Template.Callback('close')
    def close(self, *args):
        return super().close()

    @Gtk.Template.Callback('on_delete_event')
    def on_delete_event(self, window, event):
        validated = True
        empty = True
        for (field_name, _, widget_class) in self.REQUIRES:
            widget = getattr(self, field_name.lower())
            if not widget.get_text():
                self.requires_set_vaild(widget, False)
                validated = False
            else:
                self.requires_set_vaild(widget, True)
                empty = False
        for row in self.slaves_model:
            if not row[0]:
                row[1] = 'red'
                validated = False
            elif row[1]:
                row[1] = None
                empty = False
        if self.is_creating and empty:
            return
        if not validated:
            return True
        return self.on_close(window, event)

    @staticmethod
    def requires_set_vaild(widget, status):
        widget.set_icon_from_icon_name(
            Gtk.EntryIconPosition.SECONDARY, None if status else 'dialog-error')


class GroupDialog(EditDialog):
    REQUIRES = (
        ('Name', _('Name'), Gtk.Entry),
        ('Link', _('Link'), FileEntry),
    )
    SLAVES = REQUIRES

    def __init__(self, group=None, *args, **kwargs):
        self.group = group

        if isinstance(self.group, alternative.Group):
            self.set_title(_('Edit group - {}').format(self.group.name))
            self.name.set_text(self.group.name)
            self.link.set_text(self.group.link)
            for slave_name in self.group[1:]:
                self.slaves_model.append(
                    (slave_name, None, self.group[slave_name], None))
        else:
            self.set_title(_('Add group'))
            self.new_group_warning.show()

    @property
    def is_creating(self):
        return not isinstance(self.group, alternative.Group)

    def on_close(self, window, event):
        main_instance = self.get_transient_for().main_instance
        name = self.name.get_text()
        reload_groups_p = False
        slaves_diff = {}

        if self.group is None:
            self.group = alternative.Group(name, create=True)
            main_instance.alt_db.add(self.group)
            reload_groups_p = True
        elif name != self.group.name:
            slaves_diff[self.group.name] = name
            main_instance.alt_db.move(self.group.name, name)
            reload_groups_p = True
        self.group.link = self.link.get_text()

        slaves = set()
        for i, row in enumerate(self.slaves_model):
            new_slave = row[0]
            if self.group[i + 1] != new_slave:
                slaves_diff[self.group[i + 1]] = new_slave
            self.group[new_slave] = row[2]
            slaves.add(new_slave)
        for old_slave in self.group[1:]:
            if old_slave not in slaves:
                del self.group[old_slave]
        for option in self.group.options:
            option.update({
                slaves_diff[k]: v for k, v in option.items() if k in slaves_diff
            })

        if reload_groups_p:
            main_instance.load_groups()
        else:
            main_instance.load_options()
        main_instance.on_change()


class OptionDialog(EditDialog):
    REQUIRES = (
        ('Path', _('Path'), FileEntry),
        ('Priority', _('Priority'), Gtk.SpinButton),
    )
    SLAVES = (
        ('Name', _('Name'), Gtk.Entry),
        ('Path', _('Path'), FileEntry),
    )

    def __init__(self, option=None, group=None, *args, **kwargs):
        self.group = group
        self.option = option

        self.slaves_edit.hide()
        self.priority.set_adjustment(
            Gtk.Adjustment(0, -(1 << 31), 1 << 31, 1, 10, 0))
        if isinstance(self.option, alternative.Option):
            self.set_title(
                _('Edit option - {}').format(self.option[self.group.name]))
            self.path.set_text(self.option[self.group.name])
            self.priority.set_value(self.option.priority)
            for slave_name in self.group[1:]:
                self.slaves_model.append(
                    (slave_name, None, self.option[slave_name], None))
        else:
            self.set_title(_('Add option'))
            self.priority.set_value(0)
            for slave_name in self.group[1:]:
                self.slaves_model.append(
                    (slave_name, None, None, None))

    @property
    def is_creating(self):
        return not isinstance(self.option, alternative.Option)

    def on_close(self, window, event):
        main_instance = self.get_transient_for().main_instance
        if self.option is None:
            self.option = alternative.Option()
            self.group.options.append(self.option)
        for row in self.slaves_model:
            if row[2]:
                self.option[row[0]] = row[2]
            elif row[0] in self.option:
                del self.option[row[0]]
        path = self.path.get_text()
        self.option[self.group.name] = path
        self.option.priority = self.priority.get_value_as_int()
        main_instance.load_options()
        main_instance.on_change()


def advanced(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        if not self.edit_warning_show_check.get_active() or \
                self.edit_warning.run() != Gtk.ResponseType.CANCEL:
            return f(self, *args, **kwargs)
    return wrapper


icontheme = Gtk.IconTheme.get_default()
STATUS_ICONS = []
for icon_name in ('dialog-ok', 'dialog-error'):
    try:
        icon = icontheme.load_icon(icon_name, 8, 0)
    except GLib.Error:
        icon = None
    STATUS_ICONS.append(icon)
STATUS_ICONS.append(None)
del icon
del icontheme


class MainWindow(object):
    delay_mode = False
    group_cleaning = False
    group_filter_pattern = ''

    def __init__(self, app, paths={}, group=None):
        '''Load alternative database and fetch objects from the builder.'''
        self.paths = paths
        self.group = alternative.Group(group, create=True) if group else None
        self.use_polkit = app.use_polkit if app else \
            os.getuid() and bool(shutil.which('pkexec'))
        self.debug = app.debug

        # glade XML
        self.builder = Gtk.Builder.new_from_file(
            get_data_path('glade/galternatives.glade'))
        for widget_id in {
            # main window
            'main_window', 'main_accelgroup',
            'pending_box', 'groups_tv',
            'group_find_btn', 'group_find_entry', 'groups_tv_filter',
            'group_icon', 'alternative_label',
            'link_label', 'description_label',
            'status_switch', 'options_tv', 'options_column_package',
            'options_menu',
            # dialogs and messages
            'preferences_dialog',
            'edit_warning', 'edit_warning_show_check',
            'confirm_closing', 'commit_failed', 'results_tv'
        }:
            setattr(self, widget_id, self.builder.get_object(widget_id))
        self.main_window.set_application(app)
        self.main_window.set_icon_from_file(LOGO_PATH)
        self.main_window.main_instance = self

        # save placeholder text strings
        self.empty_group = \
            alternative.Group(self.alternative_label.get_text(), create=True)
        self.empty_group[self.empty_group.name] = self.link_label.get_text()
        self.empty_group.description = self.description_label.get_text()
        self.empty_group._current = False

        # signals
        self.builder.connect_signals(self)
        # actions
        self.options_menu.insert_action_group('win', self.main_window)
        self.actions = {}
        for name, activate in {
            ('preferences', lambda *args: self.preferences_dialog.show()),
            ('quit', self.on_quit),
            ('group.add', self.add_group),
            ('group.edit', self.edit_group),
            ('group.remove', self.remove_group),
            ('group.find', self.find_group),
            ('option.add', self.add_option),
            ('option.edit', self.edit_option),
            ('option.remove', self.remove_option),
            ('change.save', self.on_save),
            ('change.reload', self.load_db),
        }:
            action = Gio.SimpleAction(name=name)
            action.connect('activate', activate)
            self.main_window.add_action(action)
            self.actions[name] = action
        for name in {'delay_mode', 'query_package', 'use_polkit'}:
            action = Gio.SimpleAction.new_stateful(
                name, None, GLib.Variant('b', getattr(self, name)))

            def on_action_toggle_func(name):
                def on_action_toggle(action, value):
                    action.set_state(value)
                    setattr(self, name, value.get_boolean())

                return on_action_toggle

            action.connect('change-state', on_action_toggle_func(name))
            self.main_window.add_action(action)
        # workaround https://stackoverflow.com/questions/19657017
        self.builder.get_object('group_add_btn').get_child().add_accelerator(
            'clicked', self.main_accelgroup,
            *Gtk.accelerator_parse('Insert'), Gtk.AccelFlags.VISIBLE)
        self.builder.get_object('group_edit_btn').get_child().add_accelerator(
            'clicked', self.main_accelgroup,
            *Gtk.accelerator_parse('Return'), Gtk.AccelFlags.VISIBLE)
        self.builder.get_object('group_remove_btn').get_child().add_accelerator(
            'clicked', self.main_accelgroup,
            *Gtk.accelerator_parse('Delete'), Gtk.AccelFlags.VISIBLE)
        self.group_find_btn.get_child().add_accelerator(
            'clicked', self.main_accelgroup,
            *Gtk.accelerator_parse('<Control>f'), Gtk.AccelFlags.VISIBLE)
        # model filter
        self.groups_tv_filter.set_visible_func(self.group_filter)

        self.load_db()

    hide_on_delete = staticmethod(hide_on_delete)
    reset_dialog = staticmethod(reset_dialog)

    def show(self):
        '''Show the main window. Pretend itself as Gtk.Window.'''
        # Correct the display name.
        # Ref: https://stackoverflow.com/questions/9324163/
        # how-to-set-application-title-in-gnome-shell
        self.main_window.set_wmclass(INFO['program_name'], PACKAGE)
        return self.main_window.show()

    def destroy(self):
        return self.main_window.destroy()

    @property
    def has_unsaved(self):
        '''Whether there are unsaved changes'''
        return self.pending_box.get_visible()

    # config actions begin #

    @stateful_property(False)
    def query_package(self, value):
        self.options_column_package.set_visible(value)
        if value:
            self.load_options_pkgname()
        return value

    def load_config(self, widget=None):
        for option in alternative.Alternative.PATHS:
            self.builder.get_object(option + '_chooser').set_filename(
                getattr(self.alt_db, option))

    def on_preferences_dialog_response(self, widget, response_id):
        self.paths = {
            option: self.builder.get_object(option + '_chooser').get_filename()
            for option in alternative.Alternative.PATHS
        }
        if any(getattr(self.alt_db, option) != path
               for option, path in self.paths.items()):
            self.load_db()

    # config actions end #

    # commit actions begin #

    def on_quit(self, *args):
        '''Check for unsaved changes before quitting.'''
        if self.has_unsaved:
            response_id = self.confirm_closing.run()
            if response_id == Gtk.ResponseType.CANCEL:
                return True
            if response_id == Gtk.ResponseType.NO:
                return self.do_quit()
            if response_id == Gtk.ResponseType.YES:
                self.do_save()
                return self.has_unsaved
        self.do_quit()

    def do_quit(self, *args):
        '''Close the main window.'''
        self.main_window.destroy()

    def on_save(self, *args):
        self.do_save()

    def do_save(self, diff_cmds=None, autosave=False):
        '''Save changes.'''
        if diff_cmds is None:
            diff_cmds = self.alt_db.compare(self.alt_db_old)
        self.main_window.set_sensitive(False)
        if self.debug > 1:
            logger.debug(f'submit commands {diff_cmds}{" using polkit" if self.use_polkit else""}')
        threading.Thread(target=lambda: GObject.idle_add(
            self.do_save_callback, diff_cmds, autosave, *self.alt_db.commit(
                diff_cmds,
                'pkexec' if self.use_polkit else None))
        ).start()

    def do_save_callback(self, diff_cmds, autosave, returncode, results):
        self.main_window.set_sensitive(True)
        if returncode:
            # failed
            model = self.results_tv.get_model()
            model.clear()
            for cmd, result in zip_longest(friendlize(diff_cmds), results):
                it = model.append(None, (
                    STATUS_ICONS[result.returncode != 0 if result else 2],
                    cmd[0]))
                for info in cmd[1:]:
                    model.append(it, (
                        None, '    ' + info))
                if result:
                    model.append(it, (
                        None, '  ' + _('Run command: ') + ' '.join(result.cmd)))
                    out = result.out.rstrip()
                    if out:
                        model.append(it, (None, '  ' + out))
                    err = result.err.rstrip()
                    if err:
                        model.append(it, (None, '  ' + err))
            self.commit_failed.show_all()

        if not returncode and not autosave:
            # succeeded and other fields may also be changed, flush the gui
            self.load_db()
        else:
            # succeeded and only `current' changed, do a quick save
            # or failed, keep everything
            self.alt_db_old = alternative.Alternative(**self.paths)
            self.on_change()

    def save_and_quit(self, *args, **kwargs):
        self.do_save()
        self.do_quit()

    def load_db(self, *args):
        self.alt_db = alternative.Alternative(**self.paths)
        self.alt_db_old = deepcopy(self.alt_db)
        self.pending_box.hide()
        self.load_groups()

    # commit actions end #

    # detail windows actions begin #
    # TODO: windows need a good way for handling (group destruction)

    @advanced
    def add_group(self, widget, data):
        self.show_group_window(None)

    @advanced
    def edit_group(self, widget, data):
        self.show_group_window(self.group)

    @advanced
    def remove_group(self, widget, data):
        del self.alt_db[self.group.name]
        self.load_groups()
        self.on_change()

    def show_group_window(self, group):
        window = GroupDialog(group, transient_for=self.main_window)
        window.present()

    def find_group(self, widget, data):
        if self.group_find_btn.get_active():
            self.group_find_entry.show()
            self.group_find_entry.grab_focus()
        else:
            self.group_find_entry.hide()
            self.group_find_entry.set_text('')

    def on_group_find_entry_changed(self, widget):
        self.group_filter_pattern = widget.get_text()
        self.group_cleaning = True
        self.groups_tv_filter.refilter()
        self.group_cleaning = False
        self.click_group()

    def on_group_find_entry_key_release_event(self, widget, event):
        if event.keyval == Gdk.KEY_Escape:
            self.group_find_btn.set_active(False)
            self.find_group(None, None)

    def group_filter(self, model, iter, data):
        return self.group_filter_pattern in model[iter][0]

    @advanced
    def add_option(self, widget, data):
        self.show_option_window(self.group, None)

    @advanced
    def edit_option(self, widget, data):
        self.show_option_window(self.group, self.option)

    @advanced
    def remove_option(self, widget, data):
        self.load_options()
        self.on_change()

    def show_option_window(self, group, option):
        window = OptionDialog(option, group, transient_for=self.main_window)
        window.present()

    # detail windows actions end #

    # main window actions begin #

    def load_groups(self):
        # load alternative group into groups_tv
        treeview = self.groups_tv
        selection = treeview.get_selection()
        modelfilter = treeview.get_model()
        model = modelfilter.get_model()

        self.group_cleaning = True
        model.clear()
        if self.group_find_btn.get_active():
            self.group_find_btn.set_active(False)
        self.group_cleaning = False

        next_group = None
        for group_name in sorted(self.alt_db):
            it = model.append((group_name, ))
            if self.group and self.group.name == group_name:
                # clear self.group first to prevent same-tab reloading problem
                self.group = None
                path = model.get_path(it)
                treeview.set_cursor(path)
                # by this time self.group has been changed
                # disable this code block
                next_group = self.group
                self.group = None
        self.group = next_group

        if self.group is None:
            # disable edit/remove buttons
            self.actions['group.edit'].set_enabled(False)
            self.actions['group.remove'].set_enabled(False)
            # clear options_tv
            self.group = self.empty_group
            self.load_options()
            self.description_label.set_text(self.empty_group.description)
            self.group = None

    def click_group(self, widget=None):
        '''Load options for selected alternative group into options_tv.'''
        if self.group_cleaning:
            return
        if widget is None:
            widget = self.groups_tv.get_selection()
        model, it = widget.get_selected()
        if it is None:
            if self.group:
                for row in model:
                    if row[0] == self.group.name:
                        it = row.iter
                        path = model.get_path(it)
                        self.group_cleaning = True
                        self.groups_tv.set_cursor(path)
                        self.group_cleaning = False
                        break
            return
        group = self.alt_db[model.get_value(it, 0)]
        if group == self.group:
            return

        # enable buttons
        if not self.group:
            self.actions['group.edit'].set_enabled(True)
            self.actions['group.remove'].set_enabled(True)
        # save current group
        self.group = group
        self.load_options()

    def load_options(self):
        if self.group is None:
            return

        # set the name of the alternative to the information area
        name, description, icon = altname_description(self.group.name)
        self.alternative_label.set_text(name)
        self.link_label.set_text(self.group.link)
        self.description_label.set_text(description)
        self.group_icon.set_from_icon_name(
            icon, self.group_icon.get_icon_name()[1])
        self.status_switch.set_active(self.group.status)

        # set columns
        self.options_tv.get_column(1).set_title(self.group.name)
        for i in range(4, self.options_tv.get_n_columns()):
            self.options_tv.remove_column(self.options_tv.get_column(4))
        for i in range(1, len(self.group)):
            column = Gtk.TreeViewColumn(
                self.group[i], Gtk.CellRendererText(), text=i + 3)
            column.set_resizable(True)
            self.options_tv.append_column(column)

        # load options_liststore
        options_model = Gtk.ListStore(
            bool, int, str, *(str, ) * len(self.group))
        self.options_tv.set_model(options_model)
        for option in self.group.options:
            options_model.append((
                option == self.group.current,
                option.priority,
                None,
            ) + option.paths(self.group))
        if self.query_package:
            self.load_options_pkgname()

    def load_options_pkgname(self):
        for record in self.options_tv.get_model():
            if record[2]:
                break
            record[2] = query_package(record[3])

    def load_options_current(self):
        self.options_tv.get_model().foreach(
            lambda model, path, it: model.set(
                it, 0, self.group.current == self.group.options[path[0]]))

    def change_status(self, widget, gparam):
        '''
        Handle click on status_switch.
        When current status is auto, it will block the click action.
        '''
        widget.set_sensitive(not widget.get_active())
        if widget.get_active():
            self.select_option()

    def click_option(self, widget, path):
        '''Handle click on radio buttons of options.'''
        self.select_option(int(path))

    def select_option(self, index=None):
        if index is None:
            if self.group.status:
                return
        else:
            if not self.group.status and \
                    self.group.current == self.group.options[index]:
                return
        self.group.select(index)
        self.load_options_current()
        self.status_switch.set_active(self.group.status)
        self.on_change(not self.delay_mode)

    def on_options_tv_button_press_event(self, treeview, event):
        if event.button == 3 and self.group:
            pthinfo = treeview.get_path_at_pos(int(event.x), int(event.y))
            self.option = pthinfo and self.group.options[pthinfo[0][0]]
            pthinfo = bool(pthinfo)
            self.actions['option.edit'].set_enabled(pthinfo)
            self.actions['option.remove'].set_enabled(pthinfo)
            self.options_menu.popup(None, None, None, None,
                                    event.button, event.time)

    def on_change(self, autosave=False):
        diff_cmds = self.alt_db.compare(self.alt_db_old)
        if autosave and len(diff_cmds) == 1:
            self.do_save(diff_cmds, autosave=True)
            return
        if diff_cmds:
            self.pending_box.show()
        else:
            self.pending_box.hide()

    # main window actions end #


class AboutDialog(Gtk.AboutDialog):
    '''About dialog of the application.'''
    def __init__(self, **kwargs):
        kwargs.update(INFO)
        if 'license_type' in kwargs and isinstance(kwargs['license_type'], str):
            if hasattr(Gtk.License, kwargs['license_type']):
                kwargs['license_type'] = \
                    getattr(Gtk.License, kwargs['license_type'])
            else:
                logger.warn("`license_type' incorrect!")
        super().__init__(
            logo=LOGO_PATH and GdkPixbuf.Pixbuf.new_from_file_at_scale(
                LOGO_PATH, xdg.Config.icon_size, -1, True),
            translator_credits=_('translator_credits'),
            **kwargs)
        self.connect('response', hide_on_delete)
        self.connect('delete-event', hide_on_delete)



#!/usr/bin/env python3
from galternatives import PACKAGE, INFO

from glob import glob
import os
import sys
from setuptools import setup


if len(sys.argv) > 1:
    if sys.argv[1] == 'build' or sys.argv[1] == 'install':
        os.system('make -C resources')
    elif sys.argv[1] == 'clean':
        os.system('make -C resources clean')

if __name__ == '__main__':
    setup(
        name=PACKAGE,
        version=INFO['version'],
        license='GPL2+',
        description='Manager for the alternatives system',
        long_description='A GUI to help the system administrator to choose '
                         'what program should provide a given service.',
        author='Gustavo Noronha Silva',
        author_email='kov@debian.org',
        url=INFO['website'],
        scripts=['resources/galternatives'],
        packages=[PACKAGE],
        data_files=[
            ('share/applications', glob('resources/*.desktop')),
            ('share/galternatives/glade',
             glob('resources/glade/*.glade') + glob('resources/glade/*.ui')),
            ('share/galternatives/descriptions',
             glob('resources/descriptions/*.desktop')),
            ('share/icons/hicolor/48x48/apps', glob('resources/icons/*.png')),
            ('share/icons/hicolor/scalable/apps', glob('resources/icons/*.svg')),
            ('share/metainfo', glob('resources/*.metainfo.xml')),
        ] + [
            ('share/locale/{}/LC_MESSAGES'.format(locale), [
                'resources/locale/{}/LC_MESSAGES/galternatives.mo'.format(
                    locale)
            ]) for locale in os.listdir('resources/locale')
        ] if os.path.isdir('resources/locale')
        else []  # deal with `setup.py clean'
    )
